export type tasks = {
	Turnstile: {
		params: {
			siteKey: string,
			pageUrl: string,
			action?: string,
			cData?: string,
			proxy?: string
		},
		result: {
			token: string
		}
	},
	ReCaptcha2: {
		params: {
			siteKey: string,
			pageUrl: string,
			proxy?: string
		},
		result: {
			token: string
		}
	},
	FunCaptcha: {
		params: {
			siteKey: string,
			pageUrl: string,
			data: Record<string, string>
			serviceUrl?: string,
			proxy?: string
		},
		result: {
			token: string
		}
	}
}

export type methods = {
	getBalance: {
		request: {},
		response: {
			balance: string
		}
	},
	createTask: {
		request: {
			type: keyof tasks,
			data: tasks[keyof tasks]['params']
		},
		response: Task
	},
	getTask: {
		request: {
			id: number
		},
		response: Task
	}
}

export type events = {
	taskUpdate: Task,
	tokenReissue: {}
}

export type TaskState = 'queued' | 'processing' | 'error' | 'success'

export type Task<T extends keyof tasks = keyof tasks, S extends TaskState = TaskState> = {
	id: number,
	type: T,
	creationDate: string,
	cost: string
	state: S,
} & (S extends 'success'
	? { result: tasks[T]['result'] }
	: { result: null })